#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Handler for events which are issued when configuration is changed."""

from iobase.xmldata import (XmlConfNode, ANY_CHILD_NAME)
from ionas import (request,)
from libevh import (evhcmd, util)
from pal_info import (PAL_CONFFILE_PATH, PAL_CLONE_PATH, PAL_REPLICATION_PATH,
                        Results, ReqTypes)
from pkgaddon.evh_hclone_config import (CLONE_CONF_FILE, SHARES_CONF_FILE)


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

SHARES_CATEGORY = "shares"
ADD_ACTION = "add"

CLONE_NAME = "clone"
MASTER_NAME = "master"
SLAVE_PATH = MASTER_NAME + "/slave"
HOST_PATH = SLAVE_PATH + "/host"
PASSWD_PATH = SLAVE_PATH + "/passwd"
ADDITIONALINFO_NAME = "additionalinfo"
HOST_NAME = "host"
PASSWD_NAME = "passwd"
SHARES_NAME = "shares"
SHARE_NAME = "share"
USERNAME_NAME = "username"
REPLICATION_USER = "admin"

EVENT_SHARE_NAME_PATH = "shares/share/name"
SHARE_XPATH_TMPL = """sharelist/share[name="%s"]"""
UNDELETABLE_ATTR = "undeletable"
REPLICATION_NAME = "replication"

IMMEDIATE_OPT = "immediate"
TRUE_TEXT = "true"


class EvhCloneConfChange(evhcmd.EvhCmd):
    """Event handler for the Clone packge."""

    def __load_config(self, config_file):
        """Load clone configuration XML data."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                PAL_CONFFILE_PATH, config_file).get()

    def __save_config(self, config_file, config_xml):
        """Load clone configuration XML data."""

        return self.send_entry_with_file(
                                request.Entry(ReqTypes.SAVE, config_xml),
                                PAL_CONFFILE_PATH, config_file)

    def __is_sending_enabled(self, clone_config):
        """Check if the sending the landiskinfo to slave is enabled."""

        try:
            if clone_config.get_node(MASTER_NAME).is_enabled():
                return True
        except:
            pass

        return False

    def __does_share_exist(self, eventinfo):
        """Check if the share specified in the "evnetinfo" exists."""

        try:
            share_config = self.__load_config(SHARES_CONF_FILE)
            share_name = eventinfo.get_node(EVENT_SHARE_NAME_PATH).get_text()
            share_config.get_node(SHARE_XPATH_TMPL % share_name)
            return True
        except:
            pass

        return False

    def __set_replication(self, share):
        """Set "share" to the target of replication."""

        shares = XmlConfNode(SHARES_NAME)
        shares.append_child(share.get_clone())
        return self.send_entry(request.Entry(ReqTypes.SET, shares),
                                PAL_REPLICATION_PATH)

    def __try_enable_replication(self, eventinfo, clone_config):
        """Enable share replication."""

        if not self.__is_sending_enabled(clone_config):
            return

        try:
            share_config = self.__load_config(SHARES_CONF_FILE)
            share_name = eventinfo.get_node(EVENT_SHARE_NAME_PATH).get_text()
            share = share_config.get_node(SHARE_XPATH_TMPL % share_name)
            if share.is_set_to_true(UNDELETABLE_ATTR):
                return
        except:
            return

        replication = share.get_node(REPLICATION_NAME)
        replication.delete_children(ANY_CHILD_NAME)
        replication.enable()
        setting_list = [
                (HOST_NAME, clone_config.get_node(HOST_PATH).get_text()),
                (PASSWD_NAME,
                        clone_config.get_node(PASSWD_PATH).get_text()),
                (SHARE_NAME, share_name),
                (USERNAME_NAME, REPLICATION_USER),
            ]

        for setting in setting_list:
            node = XmlConfNode(setting[0])
            node.set_text(setting[1])
            replication.append_child(node)

        response_entry = self.__set_replication(share)
        if response_entry.get_result() != Results.OK:
            raise util.RequestError(response_entry.get_result())
        self.__save_config(SHARES_CONF_FILE, share_config)

    def __try_send_landiskinfo(self, eventinfo, clone_config, immediate,
            additionalinfo=None):
        """Send the landiskinfo XML data to the Slave."""

        if not self.__is_sending_enabled(clone_config):
            return

        send_entry = request.Entry(ReqTypes.SEND, CLONE_NAME)
        send_entry.get().append_child(clone_config.get_node(SLAVE_PATH))

        if additionalinfo != None:
            additional = XmlConfNode(ADDITIONALINFO_NAME)
            additional.append_child(additionalinfo)
            send_entry.get().append_child(additional)

        if immediate:
            send_entry.set_options({IMMEDIATE_OPT: TRUE_TEXT})

        response_entry = self.send_entry(send_entry, PAL_CLONE_PATH)
        if response_entry.get_result() != Results.OK:
            raise util.RequestError(response_entry.get_result())

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            immediate = False
            additionalinfo = None
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            clone_config = self.__load_config(CLONE_CONF_FILE)
            if (eventinfo.get_attr_value(CATEGORY_ATTR) == SHARES_CATEGORY and
                    eventinfo.get_attr_value(ACTION_ATTR) == ADD_ACTION):

                if eventinfo.is_set_to_true(PREVIOUS_ATTR):
                    if self.__does_share_exist(eventinfo):
                        entry.set_result(Results.OK)
                        return entry
                else:
                    try:
                        self.__try_enable_replication(eventinfo, clone_config)
                        entry.set_result(Results.OK)
                        return entry
                    except:
                        self.exception("Enable replication fail")
                        entry.set_result(Results.SEND_ERROR)
                        return entry

                immediate = True
                additionalinfo = eventinfo.get_node(ANY_CHILD_NAME)

            elif eventinfo.is_set_to_true(PREVIOUS_ATTR):
                entry.set_result(Results.OK)
                return entry

            try:
                self.__try_send_landiskinfo(eventinfo, clone_config, immediate,
                                            additionalinfo)
            except:
                self.exception("Send Error")
                entry.set_result(Results.SEND_ERROR)
                return entry
        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhCloneConfChange(option=evhcmd.EvhCmdOption()).handle_request()
